#ifdef _WINDOWS
#define WINDOWS
#endif

#include "ac_plugin.h"
#include <windows.h>
#include <stdio.h>

//---------------------------------------------------------------------------------------
void ddh_vert2clip_go()
{
   List *vertlist = ac_selection_get_vertices_all();
   if( vertlist == NULL )
   {
      return;
   }
   int numverts = list_count( vertlist );
   display_message( "Copying %d vertices' positions to the clipboard", numverts );
   // Get size of string
   char buffer[1024];
   size_t totlen = 0;
   for( List *pv = vertlist; pv != NULL; pv = pv->next )
   {
      Vertex *v = ( Vertex * )( pv->data );
      sprintf( buffer, "%g\t%g\t%g\n", v->x, v->y, v->z );
      totlen += strlen( buffer );
   }
   // Push the vertex list to a string
   char *outbuf = new char[totlen];
   char *p = outbuf;
   for( List *pv = vertlist; pv != NULL; pv = pv->next )
   {
      Vertex *v = ( Vertex * )( pv->data );
      sprintf( p, "%g\t%g\t%g\n", v->x, v->y, v->z );
      p += strlen( p );
   }
   // Copy to clipboard
   int retstat = OpenClipboard( NULL );
   if( !retstat )
   {
      display_message( "vert2clip : an error occurred opening the Windows' clipboard (%d)",
                        retstat );
   }
   else
   {
      EmptyClipboard();
      HGLOBAL clipbuf = GlobalAlloc( GMEM_DDESHARE, totlen + 1 );
      char *cbuffer = ( char * )GlobalLock( clipbuf );
      strcpy( cbuffer, outbuf );
      GlobalUnlock( clipbuf );
      SetClipboardData( CF_TEXT, clipbuf );
      CloseClipboard();
   }
   // Cleanup
   delete[] outbuf;
   list_free( &vertlist );
   display_message( "%d vertices' positions copied to the clipboard", numverts );
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginInit( AC3DPluginInitData *d )
{

   static bool firstTime = true;
   if( firstTime )
   {
      ac_add_command( "ddh_vert2clip_go", ddh_vert2clip_go );
      ac_add_tools_menu_item( "Vertex positions to clipboard", "ac3d ddh_vert2clip_go",
                              "Copy the selected vertices to the Windows clipboard as tab-separated text" );
      firstTime = false;
   }

   return 0;

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginExit()
{
   return 0;
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginAbout()
{
   return( "Vertex positions to clipboard - version 1.0 - Dennis Hawthorne" );
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginInfo()
{
   return( "Vertex positions to clipboard - Copy the selected vertices to the Windows clipboard as tab-separated text" );
}
//---------------------------------------------------------------------------------------
